

#include <qapplication.h>
#include <qboxlayout.h>

#include <qmainwindow.h>
#include <qgraphicslayout.h>

#include "qlchart.h"


QLineChart::QLineChart ( QWidget *parent ): QWidget( parent )
{
    setParent( parent );

    tickYAxis = 5;
    tickXAxis = 5;
    qrXAxisStep = 1.0;


    qcvView = new QChartView ( &qcChart, this );    
    qcvView->setDisabled( true );
//    qcvView->setParent( this );
    qcvView->setRenderHint( QPainter::Antialiasing );
    
    qcChart.addSeries( &qlsSeries );
    qcChart.addAxis( &qvaXAxis, Qt::AlignBottom );
    qcChart.addAxis( &qvaYAxis, Qt::AlignLeft );

//    qlsSeries.append( 5, 1 );
    qlsSeries.setName( qsFL1 ( "Line" ) );
    qlsSeries.attachAxis( &qvaXAxis );
    qlsSeries.attachAxis( &qvaYAxis );
/*    QPen green(Qt::red);
    green.setWidth(3);
    qlsSeries->setPen(green); */

//    qvaAxisX.setLabelsFont( qfLabel );
    qvaXAxis.setTickCount( tickXAxis );
    qvaXAxis.setRange( 0, 10 );
    qvaXAxis.setReverse ( true );

//  qvaAxisY.setLabelsFont( qfLabel );
    qvaYAxis.setRange ( -5, 1 );

    qcChart.setAnimationOptions( QChart::NoAnimation );
//    qcChart.setTitle( qsFL1 ("Dynamic line") );
    qcChart.legend()->hide();
    qcChart.setMargins( QMargins( 0, 0, 0, 0 ) );
    qcChart.layout()->setContentsMargins( 0, 0, 0, 0 );
    qcChart.setBackgroundVisible( false );


    qlValue1 = new QLabel ( qcvView );
    qlDescr1 = new QLabel ( qcvView );
    qlDescr2 = new QLabel ( qcvView );

    adjustSize();
    
    
    
//    updateDescr();
    setDescrVisible ( true, true, true );
    setValue( qsFL1 ( "descr. 1" ), qsFL1 ( "descr. 2" ), qsFL1 ( "descr. 3" ) );

    charData.init();
}

QLineChart::~QLineChart()
{
}

/***************************************************************************
    set the visibility of the descriptions
 ***************************************************************************/

void QLineChart::setDescrVisible( bool visibleValue1, bool visibleDescr1, bool visibleDescr2 )
{
    qlValue1->setVisible( visibleValue1 );
    qlDescr1->setVisible( visibleDescr1 );
    qlDescr2->setVisible( visibleDescr2 );
}

/***************************************************************************
    set current value
 ***************************************************************************/

void QLineChart::setValue( QString qsValue1, QString qsDescr1, QString qsDescr2 )
{
//    if ( qsValue1 != qsFL1 ( "" ) )
        qlValue1->setText( qsValue1 );
//    if ( qsValue2 != qsFL1 ( "" ) )
        qlDescr1->setText( qsDescr1 );
//    if ( qsValue3 != qsFL1 ( "" ) )
        qlDescr2->setText( qsDescr2 );
}

/***************************************************************************
    add chart value
 ***************************************************************************/

void QLineChart::addValue( QString qsYAxisValueIn, qreal qrXAxisStepIn )
{
    bool enabOver, enabAdd, enabFirst;

    qreal minXAxis, maxXAxis;
    
    QPointF qpfPoint;

    QList<QPointF>::reverse_iterator riQlPoint, riQlSuccPoint;

    minXAxis = qvaXAxis.min();
    maxXAxis = qvaXAxis.max(); 
        
    charData.qrMinValue = qrealMAX;
    charData.qrMaxValue = qrealMIN;

    enabOver = enabAdd = false;
    enabFirst = true;

    riQlSuccPoint = charData.dataList.rend();

    for ( riQlPoint = charData.dataList.rbegin(); !enabOver || riQlPoint != charData.dataList.rend(); !enabOver ? riQlPoint++ : riQlPoint )
    {
        enabOver = riQlPoint == charData.dataList.rend();

        if ( enabFirst )
        {
            if ( riQlPoint != charData.dataList.rend() )
                qpfPoint = *riQlPoint;

            if ( qrealCompare( qpfPoint.x(), maxXAxis  ) < 0 )
                enabAdd = true;

            enabFirst = false;
        }

        if ( !enabOver )
        {
            qpfPoint = *riQlPoint;

            qpfPoint.setX ( qpfPoint.x() + qrXAxisStep );

            if ( enabAdd )
            {
                *riQlPoint = qpfPoint;

                setMinMaxValue ( nullptr, &qpfPoint, charData );
            }
            else
            {
                if ( riQlSuccPoint != charData.dataList.rend() )
                {
                    *riQlSuccPoint = qpfPoint;

                    setMinMaxValue ( nullptr, &qpfPoint, charData );
                }

                riQlSuccPoint = riQlPoint;
            }
        }
    }

    if ( enabAdd )
        charData.dataList.prepend( qpfPoint );

    if ( !charData.dataList.isEmpty() )
    {
        qpfPoint.setX ( minXAxis );
        qpfPoint.setY ( qsYAxisValueIn.toDouble() );
        charData.dataList.first() = qpfPoint;

        setMinMaxValue ( nullptr, &qpfPoint, charData );
    }


    setMinMaxValue ( &qvaYAxis, nullptr, charData );
    updateSeries( &qlsSeries );


    if ( qrealCompare ( qrXAxisStepIn, qrXAxisStep ) )
    {
        qrXAxisStep = qrXAxisStepIn;

        update();
    }

} // void addValue( qrealY qrValue, qreal qrStepX )

/***************************************************************************
    set the y-axis range
 ***************************************************************************/

void QLineChart::setMinMaxValue( QValueAxis *qvaValue,  QPointF *qpfValue, struct T_CharData &charData )
{
    qreal qrAxisMax, qrAxisMin, qrAxisMaxRif, qrAxisMinRif, qrAxisMaxNew, qrAxisMinNew;
    qreal qrRifMaxMax, qrRifMaxMin, qrRifMinMax, qrRifMinMin;
    qreal qrAxisRange, qrMaxStep, qrMinStep, qrAutoStep;

    qrAutoStep = 0.05;
 
    if ( qpfValue )
    {
        if ( qrealCompare( qpfValue->y(), charData.qrMaxValue  ) > 0 )
            charData.qrMaxValue = qpfValue->y();

        if ( qrealCompare( qpfValue->y(), charData.qrMinValue  ) < 0 )
            charData.qrMinValue = qpfValue->y();
    }

    if ( qvaValue )
    {
        qrAxisMaxNew = qrAxisMaxRif = qrAxisMax = qvaValue->max();
        qrAxisMinNew = qrAxisMinRif = qrAxisMin = qvaValue->min();
        qrAxisRange = qrAxisMax - qrAxisMin;


        if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        {
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( charData.qrMaxValue * qrAutoStep ) ) );
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMin ) > 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue *  qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange *  qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) < 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }
        } // if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        else
        {
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrAutoStep ) );
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMax ) < 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) > 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }

            if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
                qvaValue->setMax( qrAxisMaxNew );
        } // else ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )


        qrAxisMaxNew = qrAxisMaxRif;
        if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
            qvaValue->setMax( qrAxisMaxNew );
        


        if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        {
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( charData.qrMinValue * qrAutoStep ) ) );
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMin ) < 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( charData.qrMinValue *  qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( qrAxisRange *  qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) > 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }
        } // if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        else
        {
            qrRifMinMax = charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrAutoStep ) );
            qrRifMinMax = charData.qrMinValue - abs ( floor ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMax ) > 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( qrAxisRange * qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) < 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }

            if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
                qvaValue->setMin( qrAxisMinNew );
        } // else ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )


        qrAxisMinNew = qrAxisMinRif;
        if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
            qvaValue->setMin( qrAxisMinNew );



        charData.qrOldMaxValue = charData.qrMaxValue;
        charData.qrOldMinValue = charData.qrMinValue;

    } // if ( qvaValue )
} // void setMinMaxValue( QValueAxis *qvaValue,  QPointF *qpfValue, struct T_CharData &charData )

/***************************************************************************
    update chart
 ***************************************************************************/

void QLineChart::update()
{
    int sizeFontWidth, sizeFontHeight, sizeFontChart;
    int sizeFontYAxis, sizeFontXAxis, sizeFontValue1;

    qreal sizePercWidth,  sizePercHeight;
    qreal maxAxis;

    QSize qsLChart;
    QRect qrDescr2;
    QFont qfYAxis, qfXAxis, qfValue1, qfDescr;

    qsLChart = size();


    QRectF qrfPlotArea = qcChart.plotArea(); 
    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << qsLChart.width() << qsLChart.height() <<"\033[0m";


    sizePercWidth = qsLChart.width() / 480.0;
    sizePercHeight = qsLChart.height() / 480.0;

    sizeFontValue1 = 7;
    sizeFontWidth = 5 + ( sizePercWidth * 24 );
    sizeFontHeight = 5 + ( sizePercHeight  * 24 );
    sizeFontChart = ( sizeFontWidth < sizeFontHeight ) ? sizeFontWidth : sizeFontHeight;
    if ( sizeFontChart > sizeFontValue1 )
        sizeFontValue1 = sizeFontChart;

    sizeFontYAxis = ( sizeFontValue1 >= 11 ) ? ( sizeFontValue1 - 4 ) : 7;
    sizeFontXAxis = sizeFontYAxis;


//  qfDescr.setFamily();
    qfDescr.setPointSize( ( sizeFontValue1 >= 11 ) ? ( sizeFontValue1 - 1 ) : 10 );    
    qfDescr.setBold( false );
    qfDescr.setWeight( QFont::Light );
    qfDescr.setStretch( QFont::Condensed );


    tickYAxis = 5;
    tickXAxis = 5;

    maxAxis = 150.0 * qrXAxisStep  * sizePercWidth;
    maxAxis = qRound ( maxAxis / 10.0 ) * 10.0;

    if ( qrealCompare( sizePercHeight, 0.25 ) < 0 )
       tickYAxis = 3;

    if ( qrealCompare( sizePercWidth, 0.25 ) < 0 )
    {
        tickXAxis = 3;
        
        qfDescr.setPointSize( 7 );
        qfDescr.setStretch( QFont::UltraCondensed );
    }

    qfYAxis.setPointSize( sizeFontYAxis );
    qfYAxis.setBold( false );

    qfXAxis = qfYAxis;
    qfXAxis.setPointSize( sizeFontXAxis );

//  qfValue1.setFamily();
    qfValue1.setPointSize( sizeFontValue1 );
    qfValue1.setBold( false );

    qvaYAxis.setTickCount( tickYAxis );
    qvaYAxis.setLabelsFont( qfYAxis );

    qvaXAxis.setTickCount( tickXAxis );
    qvaXAxis.setRange( 0, maxAxis);
    qvaXAxis.setLabelsFont( qfXAxis );
// 
    qlValue1->setFont( qfValue1 );
    qlValue1->adjustSize();
    qlValue1->move( 0, ( ( qsLChart.height() - qlValue1->height() ) / 2.0 ) );
    qlValue1->setFixedWidth( qsLChart.width() );
    qlValue1->setAlignment( Qt::AlignHCenter );

    qlDescr1->setFont( qfDescr );
    qlDescr1->adjustSize();
    qlDescr1->move( 3, ( ( qsLChart.height() - qlDescr1->height() ) / 10.0 ) );
    qlDescr1->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr1->setAlignment( Qt::AlignHCenter );

    qrDescr2 = qlDescr1->geometry();

    qlDescr2->setFont( qfDescr );
    qlDescr2->adjustSize();
    qlDescr2->move( qrDescr2.x() + qrDescr2.width(), ( ( qsLChart.height() - qlDescr2->height() ) / 10.0 ) );
    qlDescr2->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr2->setAlignment( Qt::AlignHCenter );
}

/***************************************************************************
    update chart series
 ***************************************************************************/

void QLineChart::updateSeries( QAbstractSeries *series )
{    
    QXYSeries *qxySeries;
    
    if ( series )
    {
        qxySeries = static_cast<QXYSeries *>( series );

        qxySeries->replace( charData.dataList );
    }
}

/***************************************************************************
    widget resize event
 ***************************************************************************/

void QLineChart::resizeEvent ( QResizeEvent *qreEvent )
{
    Q_UNUSED(qreEvent);

    qcvView->setGeometry( 0, 0, width(), height() );
//    qcChart.setGeometry( -10, 0, width(), height() );

    update();
}


/***************************************************************************
    test chart
 ***************************************************************************/

testChart::testChart() : QWidget()
{    
    QString qsDescr1, qsDescr2, qsDescr3;
    
    setWindowTitle ( qsFL1 ( "Chart (only for test)" ) );
    
    QPalette palette3, palette4;
    QBrush brush3, brush4;
    
    brush3 = QBrush (QColor(255, 255, 255, 255));
    brush3.setStyle(Qt::SolidPattern);
            
    palette3.setBrush(QPalette::Active, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Inactive, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Base, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Window, brush3);
    
    brush4 = brush3;
    brush4.setColor(QColor(200, 57, 241, 255));

    palette4 = palette3;
    palette4.setBrush(QPalette::Active, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Inactive, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Base, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Window, brush4);

    qwChart = new QWidget(this);
    qwChart->setObjectName("qwChart");
    qwChart->setGeometry(QRect(0, 0, 96, 96));

    qfChart = new QFrame(qwChart);
    qfChart->setObjectName("qfChart");
//        qfChart->setGeometry(QRect(0, 0, 250, 250));
    qfChart->resize(qwChart->size());

    qfChart->setPalette(palette3);
    qfChart->setAutoFillBackground(true);
    qfChart->setFrameShape(QFrame::Shape::Panel);
    qfChart->setFrameShadow(QFrame::Shadow::Raised);

    qwChart2 = new QWidget(this);
    qwChart2->setObjectName("qwChart2");
    qwChart2->setGeometry(QRect(300, 0, 250, 250));

    qfChart2 = new QFrame(qwChart2);
    qfChart2->setObjectName("qfChart2");
//        qfChart2->setGeometry(QRect(0, 0, 250, 250));
    qfChart2->resize(qwChart2->size());

//        qfChart2->setPalette(palette4);
    qfChart2->setAutoFillBackground(true);
    qfChart2->setFrameShape(QFrame::Shape::Panel);
    qfChart2->setFrameShadow(QFrame::Shadow::Raised);
            
    
    qclChart = new QLineChart (qfChart);
    qclChart->setGeometry( 0, 0, 96, 96 );


//    QRectF qrfPlotArea = qclChart->qcChart.plotArea(); 
//    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << size().width() << size().height() <<"\033[0m";

    
    qclChart2 = new QLineChart (qfChart2);
    qclChart2->setGeometry( 0, 0, 250, 250 );

    
//    show();        
                
    resize(600, 600);

    testStepY = 0.1;
    testStepX = 0.1;
    testPosY = 0;

//    qclChart->addData( testPosY );
    qclChart2->addValue( QString::number ( testPosY ), testStepX );
    qclChart2->setValue( qsDescr1, qsDescr2, qsDescr3 );


    idTimer = startTimer( 100 );

//   setCentralWidget(qwChart3);

/*   QMainWindow * win = (QMainWindow *) QApplication::activeWindow();        

     foreach (QWidget *w, qApp->topLevelWidgets())
        if (win = qobject_cast<QMainWindow*>(w))
          break; */
}

testChart::~testChart()
{

}

void testChart::timerEvent( QTimerEvent * )
{    
    QString qsDescr1, qsDescr2, qsDescr3;
    
    static int enabKill = 0;

    if ( testPosY == 0.0 )
        testPosY = 2.0;

    testPosY += testStepY;

    if ( enabKill >= 0 && enabKill < 10 )
    {
        if ( testPosY > 3 )
        {        
            testPosY = -20.0;

            if ( enabKill == 1 )
                enabKill = 9;

            enabKill++;
        }
    }
    else if ( enabKill >= 10 && enabKill < 12 )
    {
        if ( testPosY > 30.0 )
        {
            testPosY = 9.0;
            enabKill++;
        }
    }
    else if ( enabKill < 15 )
    {
        if ( testPosY > 12.0 )
        {
            testPosY = 9.0;

            enabKill++;
        }
    }

    qsDescr1 = qsFL1( "TCTL %1 C" ).arg ( testPosY, 0, 'f', 0, QChar::fromLatin1( ' ' ) ); 

//    qclChart->addData( posY );
    qclChart2->addValue( QString::number ( testPosY ), testStepX );
    qclChart2->setValue( qsDescr1, qsDescr2, qsDescr3 );
}
