#ifndef CHART_H
#define CHART_H

#include <qstring.h>
#include <qlabel.h>

#include <qmainwindow.h>
#include <qwidget.h>

#include <qchart.h>
#include <qchartview.h>
#include <QtCharts/QAbstractAxis>
#include <QtCharts/QSplineSeries>
#include <QtCharts/QValueAxis>
#include <QtCore/QRandomGenerator>
#include <QtCore/QTimer>

#include "general.h"


#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
using namespace QtCharts;
#endif

class Chart;

class QLineChart : public QWidget
{
    Q_OBJECT
public:

    struct T_CharData
    {
        qreal qrOldMaxValue;
        qreal qrOldMinValue;
        qreal qrMaxValue;
        qreal qrMinValue;

        QList<QPointF> dataList;

        void init()
        {
            qrOldMaxValue = qrealMIN;
            qrOldMinValue = qrealMAX;
            qrMaxValue = qrealMAX;
            qrMinValue = qrealMIN;

            dataList.clear();
        }
    };

    QLineChart ( QWidget *parent );
    ~QLineChart();
    
    void setDescrVisible( bool visibleValue1, bool visibleDescr1, bool visibleDescr2 );
    void setValue( QString qsValue1, QString qsDescr1, QString qsDescr2 );
    void addValue( QString qsYAxisValueIn, qreal qrXAxisStepIn );
    void setMinMaxValue( QValueAxis *qvaValue, QPointF *qpfValue, struct T_CharData &charData );

protected:
    void resizeEvent( QResizeEvent *qreEvent ) override;

private:
    void update();
    void updateSeries( QAbstractSeries *series );

    QLabel *qlValue1;
    QLabel *qlDescr1;
    QLabel *qlDescr2;

    int tickYAxis;
    int tickXAxis;
    qreal qrXAxisStep;

    QChart qcChart;
    QChartView *qcvView;
    QLineSeries qlsSeries;
    QValueAxis qvaXAxis;
    QValueAxis qvaYAxis;

    struct T_CharData charData;
};


class testChart : public QWidget
{
   Q_OBJECT
public:
    testChart();
    ~testChart();
    
    QLineChart *qclChart;
    QLineChart *qclChart2;

    QWidget *qwChart;
    QWidget *qwChart2;
    QFrame *qfChart;
    QFrame *qfChart2;

private:

    void timerEvent( QTimerEvent * ) override;

    int idTimer;
    
    qreal testStepY;
    qreal testStepX;
    qreal testPosY;    
};

#endif
