/***************************************************************************
                          sensor.h  -  description
                             -------------------
    begin                : mi� abr 24 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SENSOR_H
#define SENSOR_H

#include <qobject.h>
#include <qstring.h>

// #include <qcstring.h>

#include "ksensorssetting.h"

/**
  *@original author Miguel Novas
  */

//

class SensorDev;

class Sensor : public QObject, public ksensorsSetting
{
  Q_OBJECT

  friend class SensorDev;

public:

    enum SensorClass { lmSensor, hdSensor, sysSensor };

    enum TempScale { dgCelsius, dgFahrenheit, dgKelvin, dgDefault, dgNoTemp };

    enum SensorType { lmTemp, lmFan, lmVoltage, sysCpuInfo, sysCpuLoad, sysRamUsed, sysSwapUsed, sysUpTime };

    struct stValMask
    {
        QString preTxt;
        QString postTxt;
        char formType;
        int width;
        int prec;

        void init()
        {
            preTxt.clear();
            postTxt.clear();
            formType = ' ';
            width = 0;
            prec = 0;
        }
    };

    struct T_ValData
    {
        double val, val1, val2, val3, val4;
        TempScale scale;
        bool adjust, enabUpdate, valueChanged;

        void init()
        {
          val = val1 = val2 = val3 = val4 = 0.0;
          scale = dgDefault;
          adjust = true;
          enabUpdate = false;
          valueChanged = false;
        }
    };

    Sensor( SensorDev *parent = nullptr, const QString &name = QString() );
    ~Sensor();

    SensorDev *getSensorDev() { return ( (SensorDev *)parent() ); };

  inline void setType(SensorType newType)    { type= newType; };
  void setValue( struct T_ValData & valData );

  void setValDescr ( QString descrIn ) { valDescr = descrIn; };
  void setDescr1 ( QString descrIn ) { descr1 = descrIn; };
  void setDescr2 ( QString descrIn ) { descr2 = descrIn; };
  
  inline SensorType  getType()              { return type; };
  
    void setShortName( const QString &name );
    QString getShortName();
    void setLongName( const QString &name );
    QString getLongName();
    void setSubName( const QString &name );
    QString getSubName();
    void setDescription( const QString &desc );
    const QString &getDescription();

    void setObjectName( const QString &name );
    
    bool getValueChanged() { return ( valSensors.valueChanged ); };
    
  inline double getValue() { return celsiusTo( valSensors.val ); };
  inline double getValue1() { return celsiusTo( valSensors.val1 ); };
  inline double getValue2() { return celsiusTo( valSensors.val2 ); };
  inline double getValue3() { return celsiusTo( valSensors.val3 ); };
  inline double getValue4() { return celsiusTo( valSensors.val4 ); };
  
  inline QString getValDescr() { return ( valDescr ); };
  inline QString getDescr1() { return ( descr1 ); };
  inline QString getDescr2() { return ( descr2 ); };
  
  inline double getValuePrevious() 			    { return celsiusTo(valPrevious);   };
  inline double getValueIdeal()             { return celsiusTo(valIdeal);      };
  inline double getValueMax()               { return celsiusTo(valMax);        };
  inline double getValueMin()               { return celsiusTo(valMin);        };
  inline double getCompensation()           { return celsiusToDiff(compensation);  };
  inline double getMultiplicator()          { return multiplicator; };  
  inline bool   getAlarm() { return ( ( valSensors.val < valMin && valMin > -1000000 ) || ( valSensors.val > valMax && valMax > -1000000 ) ); };
  inline bool   getAlarmPrevious() { return ( (valPrevious<valMin && valMin > -1000000 ) || ( valPrevious>valMax && valMax > -1000000 ) ); };
  inline bool   getMonitorize()						  { return monitorize;   };
  inline bool   isAlarmValue(double value) { value= toCelsius(value); return ( ( value < valMin && valMin > -1000000 ) || ( value > valMax && valMax > -1000000 ) ); };

  bool monitorized();
  void setMonitorized  (bool enable);
  void setValueIdeal   (double value, TempScale scale= dgDefault);
  void setValueMax     (double value, TempScale scale= dgDefault);
  void setValueMin     (double value, TempScale scale= dgDefault);
  void setCompensation (double value, TempScale scale= dgDefault);
  void setMultiplicator(double value);

  double toCelsius(double val, TempScale scale= dgDefault);
  double celsiusTo(double val, TempScale scale= dgDefault);
  double toCelsiusDiff(double val, TempScale scale= dgDefault);
  double celsiusToDiff(double val, TempScale scale= dgDefault);

  struct Sensor::stValMask getPrintMask(bool addSufix);

public Q_SLOTS:

  void readConfig();
  void writeConfig();

Q_SIGNALS:

  void valueChanged();
  
  void panelDockConfigChanged();

private:

    SensorType  type;

    QString shortName;
    QString longName;
    QString subName;

    QString description;

    struct T_ValData valSensors;
   
   QString valDescr;
   QString descr1;
   QString descr2;
   
   double     valPrevious;
   double     compensation;
   double     multiplicator;
   double     valMax;
   double     valMin;
   double     valIdeal;
   bool       monitorize;

   inline double adjustValue(double value)      { return value * multiplicator + compensation; };
   struct stValMask getSensorPrintMask(int sensorType, bool addSufix, TempScale temp);

};

#endif
