/***************************************************************************
                          sensorscfg.cpp  -  description
                             -------------------
    begin                : Mon Aug 13 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcolor.h>
// #include <qlistbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
// #include <qobjcoll.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qdebug.h>

#include <kcolorbutton.h>
#include <klocalizedstring.h>
#include <kaboutapplicationdialog.h>

#include "general.h"
#include "lmsensorsalarms.h"
#include "sensorpanel.h"
#include "lmsensordockpanel.h"
#include "panelswindow.h"
#include "sensorscfg.h"


// sensorsCfg::sensorsCfg ( QFrame *page, SensorDev *lsensors,QWidget *parent, const char *name) : sensorsCfgDesign ( lsensors, parent,name /* ,Qt::WDestructiveClose */)

sensorsCfg::sensorsCfg ( KPageDialog *kpdKSensorsCfgIn, KsensorsMain *ksensorsMainIn, SensorDev *sensorsListIn, const QString &name ) : sensorsCfgDesign ( sensorsListIn, kpdKSensorsCfgIn, name /*, Qt::WDestructiveClose */ )
{
    QFrame *page;
    QVBoxLayout *lay;
    QString qsHeader;
    QIcon qiItem;
    KPageWidgetItem *kpwiPage;

    disabSetHeader = false;

    kpdKSensorsCfg = kpdKSensorsCfgIn;
    ksensorsMain = ksensorsMainIn;
    sensorsList = sensorsListIn;

 curSensor = nullptr ;

    page = new QFrame();
    lay = new QVBoxLayout( page );

    qsHeader = qsFL1( "Header" );
    qiItem = QIcon();
    if ( sensorsList->getClass() == Sensor::lmSensor )
    {
      qsHeader = i18n( "Sensors " ) + sensorsList->getLongName() + qsFL1( " (" ) + sensorsList->getSubName() + qsFL1( ")" );
      qiItem = QIcon ( findIcon( "motherboard.png" ) );
    }
    else if ( sensorsList->getClass() == Sensor::hdSensor )
    {
        qsHeader = sensorsList->getDescription();
        qiItem = QIcon ( findIcon("harddisk.png") );
    }
    else if ( sensorsList->getClass() == Sensor::sysSensor )
    {
        qsHeader = sensorsList->getDescription();
        qiItem = QIcon ( findIcon ( "computers.png" ) );
    }

    kpwiPage = kpdKSensorsCfg->addPage ( page, sensorsList->getDescription() );
    kpwiPage->setHeader ( qsHeader );
    kpwiPage->setIcon( qiItem );

    lay->addWidget( this );

    connect( kpdKSensorsCfgIn->button( QDialogButtonBox::Apply ), SIGNAL( clicked( bool ) ), this, SLOT( slotApplyChanges() ) );


 palPanelCfg= new PaletteCfg( boxPanelPalette );
 boxPanelPalette->setAlignment( Qt::Vertical );
// boxPanelPalette->layout()->setSpacing( 6 );
// boxPanelPalette->layout()->setMargin( 12 );
/* QGridLayout *boxPanelPaletteLayout = new QGridLayout( this );
 boxPanelPaletteLayout->setAlignment( Qt::AlignTop );
 boxPanelPaletteLayout->addWidget( palPanelCfg, 0, 0 ); */

 
 connect( MainTab        ,SIGNAL(currentChanged(int)), this, SLOT(slotMainTabPageChanged(int)));
 connect( listSensors    ,SIGNAL(currentRowChanged(int)),this,SLOT(slotCurrentSensorChanged(int)) );
 //connect( buttonApply    ,SIGNAL(clicked()),this,SLOT(slotApplyChanges()));
 connect( buttonTest     ,SIGNAL(clicked()),this,SLOT(slotTestAlarm()));
 connect( editMinValue   ,SIGNAL(activated(int)),this,SLOT(slotComboMinMaxSelected(int)));
 connect( editMaxValue   ,SIGNAL(activated(int)),this,SLOT(slotComboMinMaxSelected(int)));
 
 readSensorsList();
}

sensorsCfg::~sensorsCfg()
{
}

void sensorsCfg::readSensorsList()
{
 const QObjectList *list= sensorsList->getSensors();
 if(list) {
   listSensors->clear();
   QPixmap pm;
   
   for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) 
   {
     LMSensor *sensor = (LMSensor *)(*it);
 
     if ( sensor )
     {

     switch(sensor->getType()) {
       case LMSensor::lmTemp   : pm= QPixmap( findIcon ("thermometer18x18.png") ); break;
       case LMSensor::lmFan    : pm= QPixmap( findIcon ("cooler18x18.png") );      break;
       case LMSensor::lmVoltage: pm= QPixmap( findIcon ("battery18x18.png") );     break;
       
       default:
         
         break;
     }
     
     QListWidgetItem *item = new QListWidgetItem( pm, sensor->getDescription(), listSensors, QListWidgetItem::UserType);
     Q_UNUSED(item);
          
     }
     else
       break;
     
   }

   disabSetHeader = true;
   listSensors->setCurrentRow( 0 );
   disabSetHeader = false;
 }
}


void sensorsCfg::readSensorInfo(int index)
{
    QString qsHeader;
    QString qsShowType;
    KPageWidgetItem *kpwiKSensorsCfg;

 if(index<0) return;

 Sensor *oldSensor= curSensor;

 curSensor= sensorsList->getSensor(index);

 if(!curSensor) return;

 if( oldSensor == nullptr || curSensor->getType()!=oldSensor->getType() || curSensor->getSensorDev()->getClass()!=oldSensor->getSensorDev()->getClass() )
   switch(curSensor->getType()) {
     case LMSensor::lmTemp   : GroupBoxDes->setTitle(i18n("Temperature"));
                               pixmapSensor->setPixmap( QPixmap( findIcon(
                                                                  curSensor->getSensorDev()->getClass()==Sensor::lmSensor ?
                                                                  "thermometer.png":"harddisk.png"
                                                                ) ) );
                               break;
     case LMSensor::lmFan    : GroupBoxDes->setTitle(i18n("Fan"));
                               pixmapSensor->setPixmap( QPixmap( findIcon("cooler.png") ) );
                               break;
     case LMSensor::lmVoltage: GroupBoxDes->setTitle(i18n("Voltage"));
                               pixmapSensor->setPixmap( QPixmap( findIcon("battery.png") ) );
                               break;
                               
     default:
       
       break;
   }


    kpwiKSensorsCfg = kpdKSensorsCfg->currentPage();

    if ( kpwiKSensorsCfg )
    {
        qsHeader = kpwiKSensorsCfg->header();

        if ( curSensor->getSensorDev()->getClass() == Sensor::lmSensor )
        {
          qsHeader = i18n( "Sensors ") + qsFL1( " " ) +  curSensor->getLongName() /* + "." + curSensor->getDescription() */ ;
          qsHeader.append( qsFL1( " (" ) + sensorsList->getSubName() + qsFL1( ")" ) );
        }
        else if ( curSensor->getSensorDev()->getClass() == Sensor::hdSensor )
        {
            qsHeader = i18n( "Hard Disks" ) + qsFL1( " " ) + curSensor->getLongName() + qsFL1( " " ) + curSensor->getDescription();
        }
        else if ( curSensor->getSensorDev()->getClass() == Sensor::sysSensor )
        {
            qsHeader = i18n( "System Information" ) + qsFL1( " " ) + curSensor->getDescription();
        }

        if ( !disabSetHeader )
            kpwiKSensorsCfg->setHeader( qsHeader );
    }


 editDescription ->setText(curSensor->getDescription());

 struct Sensor::stValMask vMask, vMaskV;
 QString str;

 vMask.init();
 vMask.prec = (curSensor->getType() == LMSensor::lmVoltage ? 2 : 0 );
 vMask.formType = 'f';
 
 vMaskV = curSensor->getPrintMask( true );
 
 str = qsFL1( "%1%2%3" ).arg( vMaskV.preTxt ).arg ( curSensor->getValue(), vMaskV.width, vMaskV.formType, vMaskV.prec, QChar::fromLatin1( ' ' ) ).arg ( vMaskV.postTxt );
 editValue->setText(str);

 str = qsFL1( "%1%2%3" ).arg( vMask.preTxt ).arg ( curSensor->getValueIdeal(), vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1( ' ' ) ).arg ( vMask.postTxt );
 editIdealValue->setText(str);

 str = qsFL1( "%1%2%3" ).arg( vMask.preTxt ).arg ( curSensor->getValueMin() , vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1( ' ' ) ).arg ( vMask.postTxt );
 editMinValue->setEditText(str);

 str = qsFL1( "%1%2%3" ).arg( vMask.preTxt ).arg ( curSensor->getValueMax(), vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1( ' ' ) ).arg ( vMask.postTxt );
 editMaxValue->setEditText(str);

 editSum->setText(QString::number(curSensor->getCompensation()));
 editMul->setText(QString::number(curSensor->getMultiplicator()));

 //checkShowInDock->setEnabled( curSensor->getType()==LMSensor::lmTemp );
 //GroupBoxColorDock->setEnabled( curSensor->getType()==LMSensor::lmTemp );

 CheckAllSensors->setChecked(false);

 
//  setGroupConf ( curSensor->objectName() );
    setSensorDevGroupConf ( curSensor->getShortName(), curSensor->getLongName(), curSensor->getSubName() );
    colorAlarm->setColor ( getEntryConf ( qsFL1( "colorAlarm" ), QColor(), ksensorsSetting::AutoDefault ) );
    colorNormal->setColor ( getEntryConf ( qsFL1( "colorNormal" ), QColor(), ksensorsSetting::AutoDefault ) );
    
    qsShowType = getEntryConf ( qsFL1( "show" ), qsFL1( "false" ), ksensorsSetting::AutoDefault );        
    qcbShowType->setCurrentIndex( qcbShowType->findData ( qsShowType, ( Qt::UserRole + 1 ) ) );

    checkShowInDock->setChecked ( checkShowInDock->isEnabled() && getEntryConf ( qsFL1( "showInDock" ), false, ksensorsSetting::AutoDefault ) );  

    editLaunch->setText( getEntryConf ( qsFL1( "AlarmCommand" ), QString(), ksensorsSetting::AutoDefault ) );
    editPlay  ->setText( getEntryConf ( qsFL1( "AlarmSound" ), QString(), ksensorsSetting::AutoDefault ) );
  
 switch( getEntryConf ( qsFL1( "Alarm" ), -1, ksensorsSetting::AutoDefault ) )
 {
   case ksensorsSetting::acNothing : radioAlarm1->setChecked(true); break;
   case ksensorsSetting::acSound   : radioAlarm2->setChecked(true); break;
   case ksensorsSetting::acCommand : radioAlarm3->setChecked(true); break;
 }
 palPanelCfg->readPalette( curSensor->getShortName(), curSensor->getLongName(), curSensor->getSubName() );

}


void sensorsCfg::readPreferencesInfo()
{
    if ( qgbRadioTemp )
    {
        switch(sensorsList->getTempScale()) {
        case LMSensor::dgCelsius   :  RadioC->setChecked(true); break;
        case LMSensor::dgFahrenheit:  RadioF->setChecked(true); break;
        case LMSensor::dgKelvin    :  RadioK->setChecked(true); break;
        default: 

        break;
    }
 }

    setValueSpinUpdateTime ( sensorsList->getConfUpdateInterval() );
}


void sensorsCfg::applySensorChanges()
{
    bool withChange;

    QString qsShowType;

    withChange = false;

 if(!curSensor) return;

 const QString sName( curSensor->getLongName() );

  // Apply sensor configuration
    qsShowType = qcbShowType->currentData( ( Qt::UserRole + 1 ) ).toString();
  
 curSensor->setDescription(editDescription->text());
 curSensor->setCompensation(editSum->text().toDouble());
 curSensor->setMultiplicator(editMul->text().toDouble());
 curSensor->setValueIdeal(editIdealValue->text().toDouble());
 curSensor->setValueMax(editMaxValue->currentText().toDouble());
 curSensor->setValueMin(editMinValue->currentText().toDouble());
 curSensor->setMonitorized(
                            qsShowType != qsFL1( "false" ) ||
                            checkShowInDock->isChecked() ||
                            !radioAlarm1->isChecked()
                          );

//  setGroupConf ( sName );
    setSensorDevGroupConf ( curSensor->getShortName(), curSensor->getLongName(), curSensor->getSubName() );
    setEntryConf ( qsFL1( "ShortName" ),  curSensor->getShortName() );
    setEntryConf ( qsFL1( "SubName" ),  curSensor->getSubName() );    
 // Apply show in main window configuration
    if ( setEntryConf ( qsFL1( "show" ), qcbShowType->currentData( ( Qt::UserRole + 1 ) ).toString() ) )
        withChange = true;

 // Apply dock configuration 
    setEntryConf ( qsFL1( "showInDock" ), checkShowInDock->isChecked() );
    setEntryConf ( qsFL1( "colorNormal" ), colorNormal->color() );
    setEntryConf ( qsFL1( "colorAlarm" ), colorAlarm->color() );

 
 //
 palPanelCfg->savePalette();
 //
 // Apply alarms configuration
    if( CheckAllSensors->isChecked() )
    {
        const QObjectList *list= sensorsList->getSensors();

        for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) 
        {
            Sensor *sensor = (Sensor *)(*it);

            if ( sensor )
            {
                applySensorAlarmConfig(sensor);

                sensor->writeConfig();
                ksensorsMain->updateSensorsPanelsDocks ( sensor->getLongName(), false );
            }
            else
                break;
        }
    }
    else 
    {
        applySensorAlarmConfig(curSensor);

        curSensor->writeConfig();
        ksensorsMain->updateSensorsPanelsDocks ( curSensor->getLongName(), withChange );
    }

 // This line must always be at the end of the method
 QListWidgetItem *item = listSensors->currentItem();
 item->setText( editDescription->text() );
  
}


void sensorsCfg::applySensorAlarmConfig(Sensor *sensor)
{
 const QString &name= sensor->getLongName();
 
//    setGroupConf ( name );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
 
    setEntryConf ( qsFL1( "AlarmCommand" ), editLaunch->text() );
    setEntryConf ( qsFL1( "AlarmSound" ), editPlay->text() );

 int code= ksensorsSetting::acNothing;
 if(radioAlarm1->isChecked()) code= ksensorsSetting::acNothing;
 else if(radioAlarm2->isChecked()) code= ksensorsSetting::acSound;
 else    if(radioAlarm3->isChecked()) code= ksensorsSetting::acCommand;
 
    setEntryConf ( qsFL1( "Alarm" ), code );
   
//  qDebug() << "\033[91m" << "qDebug(): sensorsCfg::applySensorAlarmConfig() " << name <<"\033[0m";
  
}


void sensorsCfg::applyPreferencesChanges()
{
    if ( qgbRadioTemp )
    {
        LMSensor::TempScale scale= LMSensor::dgCelsius;
        if(RadioK->isChecked()) scale= LMSensor::dgKelvin;
        else if(RadioF->isChecked()) scale= LMSensor::dgFahrenheit;

        sensorsList->setTempScale(scale);
    }

/* warning, the time intervals are then realigned by the function SensorDev::readConfig(),
   they should already be aligned at this point */
    sensorsList->setConfUpdateInterval( SpinUpdateTime->value() );
    sensorsList->setSensorUpdateInterval ( getSettingUpdateInterval ( qsFL1( "UpdateInterval" ), -1.0, Type::AutoDefault ) );

    sensorsList->writeConfig();
    setSensorDevGroupConf ( sensorsList->getShortName(), sensorsList->getLongName(), sensorsList->getSubName() );

    ksensorsMain->updateSensorsPanelsDocks ( sensorsList->getLongName(), false );
}


void sensorsCfg::slotCurrentSensorChanged(int currentRow)
{
   readSensorInfo(currentRow);
}

void sensorsCfg::slotApplyChanges()
{
  if (!isVisible())
	  return;

    switch(MainTab->currentIndex())
    {
    case 0:
        applySensorChanges();

        break;
    case 1:
        applyPreferencesChanges();

        break;
    }    
}

void sensorsCfg::slotAccept()
{
 slotApplyChanges();
 close();
}

void sensorsCfg::slotTestAlarm()
{
  if( radioAlarm2->isChecked() ) {
    LMSensorsAlarms::playSound( editPlay->text() );
  } else
  if( radioAlarm3->isChecked() ) {
    LMSensorsAlarms::runSensorCommand( qsFL1( "TEST" ), qsFL1( "100" ), editLaunch->text() );
  }
}

void sensorsCfg::slotMainTabPageChanged(int index)
{
  Q_UNUSED(index);
  
  switch(MainTab->currentIndex())
  {
    case 0: readSensorInfo(listSensors->currentRow());  break;
    case 1: readPreferencesInfo();                       break;
  }
}

void sensorsCfg::slotComboMinMaxSelected(int index)
{
 QComboBox *combo  = (QComboBox*)sender();
 QString       str = combo->itemText(index);
 double     percent= str.left(str.length()-1).toDouble();
 double     value  = editIdealValue->text().toDouble();

 if(value<0) percent= -percent;
 value= value + value * percent / 100;

  struct Sensor::stValMask vMask;
  
  vMask.init();
  vMask.prec = ( curSensor->getType()==LMSensor::lmVoltage ? 2 : 0 );
  vMask.formType = 'f';

  str = qsFL1( "%1%2%3" ).arg( vMask.preTxt ).arg ( value, vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1 ( ' ' ) ).arg ( vMask.postTxt );  

 combo->setEditText(str);
}

void sensorsCfg::showEvent ( QShowEvent *event )
{
  Q_UNUSED(event);
  
// qDebug() << "\033[91m" << "qDebug(): sensorsCfg::showEvent" << "event " << event << "\033[0m";

  int index = MainTab->currentIndex();
  
  if ( index >= 0 )
    slotMainTabPageChanged ( index );
}
